﻿using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 색상 콤보
    /// </summary>
    public partial class ColorCombo : UserControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 색상 변경시 - ColorChanged

        /// <summary>
        /// 색상 변경시
        /// </summary>
        public event EventHandler ColorChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 항목들 소스 의존 속성
        /// </summary>
        public static readonly DependencyProperty ItemsSourceProperty;

        /// <summary>
        /// 팝업 너비 의존 속성
        /// </summary>
        public static readonly DependencyProperty PopupWidthProperty;

        /// <summary>
        /// 팝업 높이 의존 속성
        /// </summary>
        public static readonly DependencyProperty PopupHeightProperty;

        /// <summary>
        /// 색상 의존 속성
        /// </summary>
        public static readonly DependencyProperty ColorProperty;

        /// <summary>
        /// 텍스트 의존 속성
        /// </summary>
        public static readonly DependencyProperty TextProperty;

        /// <summary>
        /// 알려진 색상들 채우기 여부 의존 속성
        /// </summary>
        public static readonly DependencyProperty FillKnownColorsProperty;

        /// <summary>
        /// 색상 그리드 사용 여부 의존 속성
        /// </summary>
        public static readonly DependencyProperty UseColorGridProperty;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 마우스 버튼 상위 여부
        /// </summary>
        private bool isMouseOverButton = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 버튼 - Button

        /// <summary>
        /// 버튼
        /// </summary>
        public Button Button
        {
            get
            {
                return this.button;
            }
        }

        #endregion
        #region 색상 리스트 - ColorList

        /// <summary>
        /// 색상 리스트
        /// </summary>
        public ColorList ColorList
        {
            get
            {
                return this.colorList;
            }
        }

        #endregion

        #region 항목들 소스 - ItemsSource

        /// <summary>
        /// 항목들 소스
        /// </summary>
        public IEnumerable ItemsSource
        {
            get
            {
                return (IEnumerable)GetValue(ItemsSourceProperty);
            }
            set
            {
                SetValue(ItemsSourceProperty, value);
            }
        }

        #endregion
        #region 팝업 너비 - PopupWidth

        /// <summary>
        /// 팝업 너비
        /// </summary>
        public double PopupWidth
        {
            get
            {
                return (double)GetValue(PopupWidthProperty);
            }
            set
            {
                SetValue(PopupWidthProperty, value);
            }
        }

        #endregion
        #region 팝업 높이 - PopupHeight

        /// <summary>
        /// 팝업 높이
        /// </summary>
        public double PopupHeight
        {
            get
            {
                return (double)GetValue(PopupHeightProperty);
            }
            set
            {
                SetValue(PopupHeightProperty, value);
            }
        }

        #endregion
        #region 색상 - Color

        /// <summary>
        /// 색상
        /// </summary>
        public Color? Color
        {
            get
            {
                return (Color?)GetValue(ColorProperty);
            }
            set
            {
                SetValue(ColorProperty, value);
            }
        }

        #endregion
        #region 텍스트 - Text

        /// <summary>
        /// 텍스트
        /// </summary>
        public string Text
        {
            get
            {
                return (string)GetValue(TextProperty);
            }
            set
            {
                SetValue(TextProperty, value);
            }
        }

        #endregion
        #region 알려진 색상들 채우기 여부 - FillKnownColors

        /// <summary>
        /// 알려진 색상들 채우기 여부
        /// </summary>
        public bool FillKnownColors
        {
            get
            {
                return (bool)GetValue(FillKnownColorsProperty);
            }
            set
            {
                SetValue(FillKnownColorsProperty, value);
            }
        }

        #endregion
        #region 색상 그리드 사용 여부 - UseColorGrid

        /// <summary>
        /// 색상 그리드 사용 여부
        /// </summary>
        public bool UseColorGrid
        {
            get
            {
                return (bool)GetValue(UseColorGridProperty);
            }
            set
            {
                SetValue(UseColorGridProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - ColorCombo()

        /// <summary>
        /// 생성자
        /// </summary>
        static ColorCombo()
        {
            ItemsSourceProperty     = RegisterDependencyProperty(null        , false, false, false, true, null                        , "ItemsSource"    , typeof(IEnumerable), typeof(ColorCombo), null);
            PopupWidthProperty      = RegisterDependencyProperty(150d        , false, false, false, true, null                        , "PopupWidth"     , typeof(double     ), typeof(ColorCombo), null);
            PopupHeightProperty     = RegisterDependencyProperty(100d        , false, false, false, true, null                        , "PopupHeight"    , typeof(double     ), typeof(ColorCombo), null);
            ColorProperty           = RegisterDependencyProperty(null        , false, false, false, true, colorPropertyChangedCallback, "Color"          , typeof(Color?     ), typeof(ColorCombo), null);
            TextProperty            = RegisterDependencyProperty(string.Empty, false, false, false, true, null                        , "Text"           , typeof(string     ), typeof(ColorCombo), null);
            FillKnownColorsProperty = RegisterDependencyProperty(false       , false, false, false, true, null                        , "FillKnownColors", typeof(bool       ), typeof(ColorCombo), null);
            UseColorGridProperty    = RegisterDependencyProperty(false       , false, false, false, true, null                        , "UseColorGrid"   , typeof(bool       ), typeof(ColorCombo), null);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ColorCombo()

        /// <summary>
        /// 생성자
        /// </summary>
        public ColorCombo()
        {
            InitializeComponent();

            SetBinding(this, ItemsSourceProperty    , this.colorList, ColorList.ItemsSourceProperty    , BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);
            SetBinding(this, ColorProperty          , this.colorList, ColorList.ColorProperty          , BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);
            SetBinding(this, TextProperty           , this.colorList, ColorList.TextProperty           , BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);
            SetBinding(this, FillKnownColorsProperty, this.colorList, ColorList.FillKnownColorsProperty, BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);
            SetBinding(this, UseColorGridProperty   , this.colorList, ColorList.UseColorGridProperty   , BindingMode.TwoWay, UpdateSourceTrigger.PropertyChanged);

            this.popup.IsOpen = true;
            this.popup.IsOpen = false;

            this.button.Click          += button_Click;
            this.popup.Opened          += popup_Opened;
            this.popup.Closed          += popup_Closed;
            this.colorList.ItemMouseUp += colorList_ItemMouseUp;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 의존 속성 등록하기 - RegisterDependencyProperty(defaultValue, affectsRender, affectsMeasure, affectsArrange, inherits, propertyChangedCallback, propertyName, propertyType, ownerType, validateValueCallback)

        /// <summary>
        /// 의존 속성 등록하기
        /// </summary>
        /// <param name="defaultValue">디폴트 값</param>
        /// <param name="affectsRender">렌더링 여향 여부</param>
        /// <param name="affectsMeasure">크기 측정 영향 여부</param>
        /// <param name="affectsArrange">배치 영향 여부</param>
        /// <param name="inherits">상속 여부</param>
        /// <param name="propertyChangedCallback">속성 변경 콜백</param>
        /// <param name="propertyName">속성명</param>
        /// <param name="propertyType">속성 타입</param>
        /// <param name="ownerType">소유자 타입</param>
        /// <param name="validateValueCallback">값 무결성 콜백</param>
        /// <returns>의존 속성</returns>
        public static DependencyProperty RegisterDependencyProperty(object defaultValue, bool affectsRender, bool affectsMeasure, bool affectsArrange, bool inherits, PropertyChangedCallback propertyChangedCallback, string propertyName, Type propertyType, Type ownerType, ValidateValueCallback validateValueCallback)
        {
            FrameworkPropertyMetadata frameworkPropertyMetadata = new FrameworkPropertyMetadata();

            frameworkPropertyMetadata.DefaultValue   = defaultValue;
            frameworkPropertyMetadata.AffectsRender  = affectsRender;
            frameworkPropertyMetadata.AffectsMeasure = affectsMeasure;
            frameworkPropertyMetadata.AffectsArrange = affectsArrange;
            frameworkPropertyMetadata.Inherits       = inherits;

            frameworkPropertyMetadata.PropertyChangedCallback += propertyChangedCallback;

            return DependencyProperty.Register(propertyName, propertyType, ownerType, frameworkPropertyMetadata, validateValueCallback);
        }

        #endregion
        #region 바인딩 설정하기 - SetBinding(targetElement, targetProperty, source, sourceProperty, bindingMode, updateSourceTrigger)

        /// <summary>
        /// 바인딩 설정하기
        /// </summary>
        /// <param name="targetElement">타겟 엘리먼트</param>
        /// <param name="targetProperty">타겟 의존 속성</param>
        /// <param name="source">소스 객체</param>
        /// <param name="sourceProperty">소스 의존 속성</param>
        /// <param name="bindingMode">바인딩 모드</param>
        /// <param name="updateSourceTrigger">소스 업데이트 트리거</param>
        public static void SetBinding(FrameworkElement targetElement, DependencyProperty targetProperty, object source, DependencyProperty sourceProperty, BindingMode bindingMode, UpdateSourceTrigger updateSourceTrigger)
        {
            Binding binding = new Binding();

            binding.Source              = source;
            binding.Path                = new PropertyPath(sourceProperty);
            binding.Mode                = bindingMode;
            binding.UpdateSourceTrigger = updateSourceTrigger;

            targetElement.SetBinding(targetProperty, binding);
        }

        #endregion
        #region 색상 속성 변경시 콜백 처리하기 - colorPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 색상 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void colorPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            ColorCombo colorCombo = dependencyObject as ColorCombo;

            colorCombo.FireColorChanged();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 구하기 - GetItem()

        /// <summary>
        /// 항목 구하기
        /// </summary>
        /// <returns>항목</returns>
        public ColorItem GetItem() 
        {
            return this.colorList.GetItem();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 색상 변경시 이벤트 발생시키기 - FireColorChanged()

        /// <summary>
        /// 색상 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireColorChanged()
        {
            if(ColorChanged != null)
            {
                ColorChanged(this, EventArgs.Empty);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 버튼 클릭시 처리하기 - button_Click(sender, e)

        /// <summary>
        /// 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void button_Click(object sender, RoutedEventArgs e)
        {
            if(this.isMouseOverButton)
            {
                this.isMouseOverButton = false;

                return;
            }

            this.popup.IsOpen = !this.popup.IsOpen;
        }

        #endregion
        #region 팝업 열리는 경우 처리하기 - popup_Opened(sender, e)

        /// <summary>
        /// 팝업 열리는 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void popup_Opened(object sender, EventArgs e)
        {
            ObservableCollection<ColorItem> collection = ItemsSource as ObservableCollection<ColorItem>;

            if(collection == null || collection.Count == 0)
            {
                PopupHeight = 16;

                return;
            }

            if(!UseColorGrid)
            {
                ListBoxItem listBoxItem = this.colorList.ItemContainerGenerator.ContainerFromIndex(0) as ListBoxItem;

                if(listBoxItem != null)
                {
                    if(!listBoxItem.ActualHeight.Equals(Double.NaN))
                    {
                        PopupHeight = Math.Min(collection.Count * listBoxItem.ActualHeight, 20 * listBoxItem.ActualHeight) + 10;
                    }
                }
            }

            ColorItem item = this.colorList.SelectedItem as ColorItem;

            if(item == null)
            {
                return;
            }

            this.colorList.ScrollIntoView(item);
        }

        #endregion
        #region 팝업이 닫히는 경우 처리하기 - popup_Closed(sender, e)

        /// <summary>
        /// 팝업이 닫히는 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void popup_Closed(object sender, EventArgs e)
        {
            Point mousePoint = Mouse.GetPosition(this.button);

            if(mousePoint.X >= 0 && mousePoint.X <= this.button.ActualWidth && mousePoint.Y >= 0 && mousePoint.Y <= this.button.ActualHeight)
            {
                this.isMouseOverButton = true;
            }
            else
            {
                this.isMouseOverButton = false;
            }
        }

        #endregion
        #region 색상 리스트 항목 마우스 UP 처리하기 - colorList_ItemMouseUp(sender, e)

        /// <summary>
        /// 색상 리스트 항목 마우스 UP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void colorList_ItemMouseUp(object sender, EventArgs e)
        {
            this.popup.IsOpen = false;
        }

        #endregion
    }
}