﻿using System.ComponentModel;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 색상 항목
    /// </summary>
    public class ColorItem : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 - PropertyChanged

        /// <summary>
        /// 속성 변경시
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 색상
        /// </summary>
        private Color? color;

        /// <summary>
        /// 텍스트
        /// </summary>
        private string text;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 색상 - Color

        /// <summary>
        /// 색상
        /// </summary>
        public Color? Color
        {
            get
            {
                return this.color;
            }
            set
            {
                if(this.color == value)
                {
                    return;
                }

                this.color = value;

                FirePropertyChanged("Color");
            }
        }

        #endregion
        #region 텍스트 - Text

        /// <summary>
        /// 텍스트
        /// </summary>
        public string Text
        {
            get
            {
                return this.text;
            }
            set
            {
                if(this.text == value)
                {
                    return;
                }

                this.text = value;

                FirePropertyChanged("Text");
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 생성자 - ColorItem()

        /// <summary>
        /// 생성자
        /// </summary>
        public ColorItem()
        {
        }

        #endregion
        #region 생성자 - ColorItem(color, text)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="color">색상</param>
        /// <param name="text">텍스트</param>
        public ColorItem(Color color, string text)
        {
            this.color = color;
            this.text  = text;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 문자열 구하기 - ToString()

        /// <summary>
        /// 문자열 구하기
        /// </summary>
        /// <returns>문자열</returns>
        public override string ToString()
        {
            return this.text;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChanged(propertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        protected void FirePropertyChanged(string propertyName)
        {
            if(PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion
    }
}