﻿using System;
using System.Collections.ObjectModel;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 색상 리스트
    /// </summary>
    public partial class ColorList : ListBox
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 색상 변경시 - ColorChanged

        /// <summary>
        /// 색상 변경시
        /// </summary>
        public event EventHandler ColorChanged;

        #endregion
        #region 항목 마우스 UP시 - ItemMouseUp

        /// <summary>
        /// 항목 마우스 UP시
        /// </summary>
        public event EventHandler ItemMouseUp;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 색상 의존 속성
        /// </summary>
        public static readonly DependencyProperty ColorProperty;

        /// <summary>
        /// 텍스트 의존 속성
        /// </summary>
        public static readonly DependencyProperty TextProperty;

        /// <summary>
        /// 알려진 색상들 채우기 여부 의존 속성
        /// </summary>
        public static readonly DependencyProperty FillKnownColorsProperty;

        /// <summary>
        /// 색상 그리드 사용 여부 의존 속성
        /// </summary>
        public static readonly DependencyProperty UseColorGridProperty;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 값 변경시 이벤트 발생 여부
        /// </summary>
        private bool canFireValueChanged = true;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 색상 - Color

        /// <summary>
        /// 색상
        /// </summary>
        public Color? Color
        {
            get
            {
                return (Color?)GetValue(ColorProperty);
            }
            set
            {
                SetValue(ColorProperty, value);
            }
        }

        #endregion
        #region 텍스트 - Text

        /// <summary>
        /// 텍스트
        /// </summary>
        public string Text
        {
            get
            {
                return (string)GetValue(TextProperty);
            }
            set
            {
                SetValue(TextProperty, value);
            }
        }

        #endregion
        #region 알려진 색상들 채우기 여부 - FillKnownColors

        /// <summary>
        /// 알려진 색상들 채우기 여부
        /// </summary>
        public bool FillKnownColors
        {
            get
            {
                return (bool)GetValue(FillKnownColorsProperty);
            }
            set
            {
                SetValue(FillKnownColorsProperty, value);
            }
        }

        #endregion
        #region 색상 그리드 사용 여부 - UseColorGrid

        /// <summary>
        /// 색상 그리드 사용 여부
        /// </summary>
        public bool UseColorGrid
        {
            get
            {
                return (bool)GetValue(UseColorGridProperty);
            }
            set
            {
                SetValue(UseColorGridProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - ColorList()

        /// <summary>
        /// 생성자
        /// </summary>
        static ColorList()
        {
            ColorProperty           = RegisterDependencyProperty(null        , false, false, false, true, colorPropertyChangedCallback          , "Color"          , typeof(Color?), typeof(ColorList), null);
            TextProperty            = RegisterDependencyProperty(string.Empty, false, false, false, true, null                                  , "Text"           , typeof(string), typeof(ColorList), null);
            FillKnownColorsProperty = RegisterDependencyProperty(false       , false, false, false, true, fillKnownColorsPropertyChangedCallback, "FillKnownColors", typeof(bool)  , typeof(ColorList), null);
            UseColorGridProperty    = RegisterDependencyProperty(false       , false, false, false, true, useColorGridPropertyChangedCallback   , "UseColorGrid"   , typeof(bool)  , typeof(ColorList), null);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ColorList()

        /// <summary>
        /// 생성자
        /// </summary>
        public ColorList()
        {
            InitializeComponent();

            SelectionChanged += ListBox_SelectionChanged;

            AddHandler(TextBlock.MouseDownEvent, new MouseButtonEventHandler(itemTextBlock_MouseDown));
            AddHandler(TextBlock.MouseUpEvent  , new MouseButtonEventHandler(itemTextBlock_MouseUp  ));
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 의존 속성 등록하기 - RegisterDependencyProperty(defaultValue, affectsRender, affectsMeasure, affectsArrange, inherits, propertyChangedCallback, propertyName, propertyType, ownerType, validateValueCallback)

        /// <summary>
        /// 의존 속성 등록하기
        /// </summary>
        /// <param name="defaultValue">디폴트 값</param>
        /// <param name="affectsRender">렌더링 여향 여부</param>
        /// <param name="affectsMeasure">크기 측정 영향 여부</param>
        /// <param name="affectsArrange">배치 영향 여부</param>
        /// <param name="inherits">상속 여부</param>
        /// <param name="propertyChangedCallback">속성 변경 콜백</param>
        /// <param name="propertyName">속성명</param>
        /// <param name="propertyType">속성 타입</param>
        /// <param name="ownerType">소유자 타입</param>
        /// <param name="validateValueCallback">값 무결성 콜백</param>
        /// <returns>의존 속성</returns>
        public static DependencyProperty RegisterDependencyProperty(object defaultValue, bool affectsRender, bool affectsMeasure, bool affectsArrange, bool inherits, PropertyChangedCallback propertyChangedCallback, string propertyName, Type propertyType, Type ownerType, ValidateValueCallback validateValueCallback)
        {
            FrameworkPropertyMetadata frameworkPropertyMetadata = new FrameworkPropertyMetadata();

            frameworkPropertyMetadata.DefaultValue   = defaultValue;
            frameworkPropertyMetadata.AffectsRender  = affectsRender;
            frameworkPropertyMetadata.AffectsMeasure = affectsMeasure;
            frameworkPropertyMetadata.AffectsArrange = affectsArrange;
            frameworkPropertyMetadata.Inherits       = inherits;

            frameworkPropertyMetadata.PropertyChangedCallback += propertyChangedCallback;

            return DependencyProperty.Register(propertyName, propertyType, ownerType, frameworkPropertyMetadata, validateValueCallback);
        }

        #endregion
        #region 색상 속성 변경시 콜백 처리하기 - colorPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 색상 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트인자</param>
        private static void colorPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            ColorList colorList = dependencyObject as ColorList;

            Color? newValue = (Color?)e.NewValue;

            colorList.SetColor(newValue);

            string text = colorList.GetText();

            colorList.Text = text;

            colorList.FireColorChanged();
        }

        #endregion
        #region 알려진 색상들 채우기 여부 속성 변경시 콜백 처리하기 - fillKnownColorsPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 알려진 색상들 채우기 여부 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void fillKnownColorsPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            ColorList colorList = dependencyObject as ColorList;

            bool newValue = (bool)e.NewValue;

            colorList.ItemsSource = null;

            colorList.Color = null;

            if(newValue)
            {
                colorList.SetKnownColorsCollection();
            }
        }

        #endregion
        #region 색상 그리드 사용 여부 속성 변경시 콜백 처리하기 - useColorGridPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 색상 그리드 사용 여부 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트인자</param>
        private static void useColorGridPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            ColorList colorList = dependencyObject as ColorList;

            bool newValue = (bool)e.NewValue;

            if(newValue)
            {
                DataTemplate dataTemplate = colorList.FindResource("DataTemplateKey2") as DataTemplate;

                colorList.ItemTemplate = dataTemplate;

                ItemsPanelTemplate itemsPanelTemplate = colorList.FindResource("ItemsPanelTemplateKey2") as ItemsPanelTemplate;

                colorList.ItemsPanel = itemsPanelTemplate;
            }
            else
            {
                DataTemplate dataTemplate = colorList.FindResource("DataTemplateKey1") as DataTemplate;

                colorList.ItemTemplate = dataTemplate;

                ItemsPanelTemplate itemsPanelTemplate = colorList.FindResource("ItemsPanelTemplateKey1") as ItemsPanelTemplate;

                colorList.ItemsPanel = itemsPanelTemplate;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 구하기 - GetItem()

        /// <summary>
        /// 항목 구하기
        /// </summary>
        /// <returns>항목</returns>
        public ColorItem GetItem()
        {
            ColorItem item = SelectedItem as ColorItem;

            return item;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 색상 변경시 이벤트 발생시키기 - FireColorChanged()

        /// <summary>
        /// 색상 변경시 이벤트 발생시키기
        /// </summary>
        protected void FireColorChanged()
        {
            if(ColorChanged != null)
            {
                ColorChanged(this, EventArgs.Empty);
            }
        }

        #endregion
        #region 항목 마우스 UP 이벤트 발생시키기 - FireItemMouseUp()

        /// <summary>
        /// 항목 마우스 UP 이벤트 발생시키기
        /// </summary>
        public void FireItemMouseUp()
        {
            if(ItemMouseUp != null)
            {
                ItemMouseUp(this, EventArgs.Empty);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private
        ////////////////////////////////////////////////////////////////////// Event

        #region 리스트 박스 선택 변경시 처리하기 - ListBox_SelectionChanged(sender, e)

        /// <summary>
        /// 리스트 박스 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void ListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if(this.canFireValueChanged)
            {
                ColorItem item = SelectedItem as ColorItem;

                if(item == null)
                {
                    Color = Colors.Transparent;

                    return;
                }

                Color = item.Color;
            }
        }

        #endregion
        #region 항목 텍스트 블럭 마우스 DOWN 처리하기 - itemTextBlock_MouseDown(sender, e)

        /// <summary>
        /// 항목 텍스트 블럭 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void itemTextBlock_MouseDown(object sender, MouseButtonEventArgs e)
        {
            this.canFireValueChanged = false;
        }

        #endregion
        #region 항목 텍스트 블럭 마우스 UP 처리하기 - itemTextBlock_MouseUp(sender, e)

        /// <summary>
        /// 항목 텍스트 블럭 마우스 UP 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void itemTextBlock_MouseUp(object sender, MouseButtonEventArgs e)
        {
            this.canFireValueChanged = true;

            Color = GetColor();

            FireColorChanged();

            FireItemMouseUp();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////// Function

        #region 색상 설정하기 - SetColor(color)

        /// <summary>
        /// 색상 설정하기
        /// </summary>
        /// <param name="color">색상</param>
        private void SetColor(Color? color)
        {
            try
            {
                SelectionChanged -= ListBox_SelectionChanged;

                if(color.Equals(null))
                {
                    SelectedItem = null;

                    return;
                }

                ObservableCollection<ColorItem> collection = ItemsSource as ObservableCollection<ColorItem>;

                if(collection == null || collection.Count == 0)
                {
                    return;
                }

                foreach(ColorItem item in collection)
                {
                    if(color.Equals(item.Color))
                    {
                        SelectedItem = item;

                        return;
                    }
                }

                SelectedItem = null;
            }
            finally
            {
                SelectionChanged += ListBox_SelectionChanged;
            }
        }

        #endregion
        #region 색상 구하기 - GetColor()

        /// <summary>
        /// 색상 구하기
        /// </summary>
        /// <returns>색상</returns>
        private Color? GetColor()
        {
            if(SelectedItem == null)
            {
                return null;
            }

            ColorItem item = SelectedItem as ColorItem;

            if(item != null)
            {
                return item.Color;
            }
            else
            {
                return null;
            }
        }

        #endregion
        #region 텍스트 구하기 - GetText()

        /// <summary>
        /// 텍스트 구하기
        /// </summary>
        /// <returns>텍스트</returns>
        private string GetText()
        {
            if(SelectedItem == null)
            {
                return string.Empty;
            }

            ColorItem item = SelectedItem as ColorItem;

            if(item != null)
            {
                return item.Text;
            }
            else
            {
                return string.Empty;
            }
        }

        #endregion
        #region 색상명 구하기 - GetColorName(originalName)

        /// <summary>
        /// 색상명 구하기
        /// </summary>
        /// <param name="originalName">원본명</param>
        /// <returns>색상명</returns>
        private string GetColorName(string originalName)
        {
            string name = originalName[0].ToString();

            for(int i = 1; i < originalName.Length; i++)
            {
                name += (char.IsUpper(originalName[i]) ? " " : string.Empty) + originalName[i].ToString();
            }

            return name;
        }

        #endregion
        #region 알려진 색상들 컬렉션 설정하기 - SetKnownColorsCollection()

        /// <summary>
        /// 알려진 색상들 컬렉션 설정하기
        /// </summary>
        private void SetKnownColorsCollection()
        {
            ObservableCollection<ColorItem> collection = new ObservableCollection<ColorItem>();

            PropertyInfo[] propertyInfoArray = typeof(Colors).GetProperties();

            for(int i = 0; i < propertyInfoArray.Length; i++)
            {
                ColorItem item = new ColorItem();

                item.Color = (Color)propertyInfoArray[i].GetValue(null, null);
                item.Text  = GetColorName(propertyInfoArray[i].Name);

                collection.Add(item);
            }

            ItemsSource = collection;
        }

        #endregion
    }
}